import {
  IsArray,
  IsBoolean,
  IsEnum,
  IsInt,
  IsNumber,
  IsOptional,
  IsString,
  Max,
  Min,
} from 'class-validator';
import { AutoMap } from '@automapper/classes';
import { Point } from '../../../geography/domain/types/point.type';
import { Schedule } from '../types/schedule.type';
import { MarginDurations } from '../types/margin-durations.type';
import { AlgorithmType } from '../types/algorithm.enum';
import { IRequestTime } from '../interfaces/time-request.interface';
import { IRequestPerson } from '../interfaces/person-request.interface';
import { IRequestGeography } from '../interfaces/geography-request.interface';
import { IRequestRequirement } from '../interfaces/requirement-request.interface';
import { IRequestAlgorithmSettings } from '../interfaces/algorithm-settings-request.interface';

export class MatchRequest
  implements
    IRequestTime,
    IRequestPerson,
    IRequestGeography,
    IRequestRequirement,
    IRequestAlgorithmSettings
{
  @IsArray()
  @AutoMap()
  waypoints: Point[];

  @IsOptional()
  @IsString()
  @AutoMap()
  departure: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  fromDate: string;

  @IsOptional()
  @AutoMap()
  schedule: Schedule;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger: boolean;

  @IsOptional()
  @IsString()
  @AutoMap()
  toDate: string;

  @IsOptional()
  @IsInt()
  @AutoMap()
  marginDuration: number;

  @IsOptional()
  @AutoMap()
  marginDurations: MarginDurations;

  @IsOptional()
  @IsNumber()
  @Min(1)
  @Max(10)
  @AutoMap()
  seatsPassenger: number;

  @IsOptional()
  @IsNumber()
  @Min(1)
  @Max(10)
  @AutoMap()
  seatsDriver: number;

  @IsOptional()
  @AutoMap()
  strict: boolean;

  @IsOptional()
  @IsEnum(AlgorithmType)
  @AutoMap()
  algorithm: AlgorithmType;

  @IsOptional()
  @IsNumber()
  @AutoMap()
  remoteness: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  useProportion: boolean;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  proportion: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  useAzimuth: boolean;

  @IsOptional()
  @IsInt()
  @Min(0)
  @Max(359)
  @AutoMap()
  azimuthMargin: number;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  maxDetourDistanceRatio: number;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  maxDetourDurationRatio: number;

  @IsOptional()
  @IsArray()
  exclusions: number[];

  @IsOptional()
  @IsInt()
  @AutoMap()
  identifier: number;
}
